#include <Adafruit_NeoPixel.h>
#define PIN 12
#define N_LEDS 8
#define btnPin 2
#define pingPin 10
#define trigPin 8
Adafruit_NeoPixel strip = Adafruit_NeoPixel(N_LEDS, PIN, NEO_GRB + NEO_KHZ800);

uint32_t green = strip.Color(0, 255, 0);
uint32_t blue = strip.Color(0, 0, 255);
uint32_t red = strip.Color(255, 0, 0); 
const int pingdelay = 100; // how many milliseconds to wait between each ping

unsigned long litterboxdelay = 90;
int buttonState = 0;
boolean presence = false;
boolean detected = false;
int detections = 0;
long cm;
int pixels = 0;

void setup() {
  Serial.begin(9600);
  strip.begin();
  pinMode(trigPin, OUTPUT);
}

void loop() {
  cm = measuredistance();
  buttonState = digitalRead(btnPin);
  if (buttonState == HIGH) {
    detections = 0;
    pixels = 0;
    strip.clear();
  } 
  Serial.print("DISTANCE ");
  Serial.print(cm); 
  Serial.println("cm");
  
  if (cm<(30)) {
    presence = true;
    delay(15000);
    cm = measuredistance();
    if (cm>(30)) {
      presence = false;
    }
  }
  else {
    presence = false;
  }
  
  if ((presence == true) && (detected==false)) {
    newdetection();
  } 
  else if ((presence == false) && (detected==true)) {
    vacated();
  }
  delay(pingdelay); // delay between pings
}

long measuredistance() {
  long cm, duration;
  pinMode(trigPin, OUTPUT);
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(5);
  digitalWrite(trigPin, LOW);
  pinMode(pingPin, INPUT);
  duration = pulseIn(pingPin, HIGH);
  cm = microsecondsToCentimeters(duration);
  if (pixels<N_LEDS) {
      strip.setPixelColor(N_LEDS-1, red); // Blink led red to show running
      strip.show();
      strip.setPixelColor(N_LEDS-1, 0); // unblink red led
      strip.show();
  }
  return cm;
}

long microsecondsToCentimeters(long microseconds) {
  // The speed of sound is 340 m/s or 29 microseconds per centimeter.
  // The ping travels out and back, so to find the distance of the object we
  // take half of the distance travelled.
  return microseconds / 29 / 2;
}

void newdetection() { 
  detections = detections + 1;
  pixels=detections;
  detected = true;
  fill(N_LEDS,green); // Fill LEDS with Green for testing cat presence detection
  Serial.print("DETECTION ");
  Serial.println(detections);
  Serial.print("DEBOUNCE ");
  Serial.print(litterboxdelay);
  Serial.println("s");
  delay(litterboxdelay*1000); // Debounce
}

void fill(int leds, uint32_t colour) { 
  for(uint16_t i=0; i<leds; i++) {
      strip.setPixelColor(i, colour); // Draw new pixel
  }
  strip.show();
 }

// cat litter box has just been vacated
void vacated() { 
  detected = false;
  strip.clear();
  Serial.println("EMPTY"); 
  if (pixels<N_LEDS) {
    fill(pixels,blue); // Blue LEDS representing no. of visits
    }
  else {
    fill(N_LEDS,red);  // Red LEDS to indicate cat litter is full
    }
}
